﻿using System.Windows.Forms;
using GisdkAPI;

namespace BatchMaps
{
    // This extension prints a batch of maps based on chosen map features.
    // It displays a list of store names and addresses from a store points
    // layer, lets you build a list of chosen stores and then, for each
    // chosen store, automatically creates a layout on your default printer
    // and adds a map scoped to the store and the area around it, along with
    // a legend and a title containing the store name.
    // A test mode is provided which displays the prepared layout without
    // actually printing it.
    // The extension currently uses the Stores layer (NYC_SPLY.CDF) in the
    // tutorial folder. You can change the extension to use your own data.

    public class Extension : IGisdkExtension
    {
        public string Title
        {
            get { return "Batch Maps"; }
        }

        public void Load(IGisdkEngine app)
        {
            dynamic gisdk = app;
            if (gisdk.GetMap() == null)
            {
                MessageBox.Show("You must open a map to use the Batch Maps extension");
                return;
            }
            try
            {
                var builder = new BatchMaps.Builder(app);
                var form = new SelectFeaturesForm(builder.ListFeatures());
                if (form.ShowDialog(app.GetMainWindow()) == DialogResult.OK)
                {
                    builder.DrawMaps(form.SelectedFeatures, form.TestMode);
                }
                form.Dispose();
            }
            catch (System.Exception e)
            {
                MessageBox.Show("Batch Maps extension encountered an exception:\n'" + e.Message + "'");
            }
        }
    }
}
