﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows.Forms;

namespace BatchMaps
{
    // Prompts user to choose points and set or clear the test mode.
    // Displays point names in a scroll list; allows user to highlight one or more
    // and copy them to a second scroll list on the right.  
    // Drop, Move Up, and Move Down allow the user to manipulate the list on the right.

    public partial class SelectFeaturesForm : Form
    {
        private IList<Feature> selectedFeatures = new BindingList<Feature>();

        public SelectFeaturesForm(IList<Feature> features)
        {
            InitializeComponent();

            availableFeaturesList.DataSource = features;
            availableFeaturesList.DisplayMember = "Name";
            availableFeaturesList.ClearSelected();

            selectedFeaturesList.DataSource = SelectedFeatures;
            selectedFeaturesList.DisplayMember = "Name";
            selectedFeaturesList.ClearSelected();

            addButton.Enabled = false;
            dropButton.Enabled = false;
            upButton.Enabled = false;
            downButton.Enabled = false;

            testMode.Checked = true;

            availableFeaturesList.SelectedIndexChanged += (s, e) =>
                addButton.Enabled = availableFeaturesList.SelectedItems.Count > 0;
            selectedFeaturesList.SelectedIndexChanged += (s, e) =>
            {
                dropButton.Enabled = selectedFeaturesList.SelectedItems.Count > 0;
                upButton.Enabled = selectedFeaturesList.SelectedItems.Count > 0 & 
                    !selectedFeaturesList.SelectedIndices.Contains(0);
                downButton.Enabled = selectedFeaturesList.SelectedItems.Count > 0 & 
                    !selectedFeaturesList.SelectedIndices.Contains(selectedFeaturesList.Items.Count-1);
            };

            addButton.Click += AddSelectedItems;
            dropButton.Click += DropItems;
            upButton.Click += (s, e) => ShiftSelectedItemsBy(-1);
            downButton.Click += (s, e) => ShiftSelectedItemsBy(1);
        }

        public IList<Feature> SelectedFeatures
        {
            get { return selectedFeatures; }
        }

        public bool TestMode { get { return testMode.Checked; } }  

        private void AddSelectedItems(object sender, EventArgs e)
        {
            foreach (Feature feature in availableFeaturesList.SelectedItems)
            {
                if (!selectedFeatures.Contains(feature))
                    selectedFeatures.Add(feature);
            }
            availableFeaturesList.ClearSelected();
            selectedFeaturesList.ClearSelected();
            okButton.Enabled = selectedFeatures.Count > 0;
        }

        private void DropItems(object sender, EventArgs e)
        {
            var features = new Feature[selectedFeaturesList.SelectedItems.Count];
            selectedFeaturesList.SelectedItems.CopyTo(features, 0);
            foreach (Feature feature in features)
            {
                selectedFeatures.Remove(feature);
            }
            selectedFeaturesList.ClearSelected();
            okButton.Enabled = selectedFeatures.Count > 0;
        }

        private void ShiftSelectedItemsBy(int shift)
        {
            var indices = new int[selectedFeaturesList.SelectedIndices.Count];
            selectedFeaturesList.SelectedIndices.CopyTo(indices, 0);
            foreach (int index in indices)
            {
                var feature = selectedFeatures[index];
                selectedFeatures.RemoveAt(index);
                selectedFeatures.Insert(index + shift, feature);
            }
            selectedFeaturesList.SelectedIndices.Clear();
            foreach (var index in indices)
            {
                selectedFeaturesList.SelectedIndices.Add(index + shift);
            }
        }
    }
}
